/*
 * FINGERPRINT.H
 * Fingerprinting engine.
 *
 * Copyright (C) 2008  Acri Emanuele <crossbower@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "fingerprint.h"

/*
 * Free fingerp struct
 */
void free_fingerp(struct fingerp *fp)
{
    free(fp->order_res);
    free(fp->h_statuscode_res);
    free(fp->h_statustext_res);
    free(fp->h_banner_res);
    free(fp->h_powered_res);
}

/*
 * Analize HTTP header order
 */
void finger_order(struct fingerp *fp, const char *path)
{
    FILE *file;
    char fpath[MAX_DATA];
    char buffer[MIN_ARG];
    char *c=NULL, *d=NULL;
    struct stat fs;
    
    snprintf(fpath, MAX_DATA-1, "%s%s", path, FDB_ORDER);
    
    if(!(file = fopen(fpath, "r"))) {
        fprintf(stderr, "Error opening signature file:%s\n", fpath);
        exit(1);
    }
    
    if(stat(fpath, &fs)!=0) {
        fprintf(stderr, "Error stat() file:%s\n", fpath);
        exit(1);
    }
    
    fp->order_res = (char *) malloc(fs.st_size + 10);
    fp->order_res[0] = '\0';
    
    // search result
    while(fgets(buffer, MIN_ARG-1, file)) {
        c = buffer;
        d = strstr(buffer, ";");
        if(d==NULL) {
            continue;
        }
        
        *d = '\0';
        d++;
        
        if(strcmp(fp->order, d)==0) {
            myncat(fp->order_res, c, fs.st_size -1);
            myncat(fp->order_res, ",", fs.st_size -1);
        }
        
        c = NULL;
        d = NULL;
    }
    
    fclose(file);
}

/*
 * Analize HTTP status code
 */
void finger_status_code(struct fingerp *fp, const char *path)
{
    FILE *file;
    char fpath[MAX_DATA];
    char buffer[MIN_ARG];
    char *c=NULL, *d=NULL;
    struct stat fs;
    
    snprintf(fpath, MAX_DATA-1, "%s%s", path, FDB_STATUSCODE);
    
    if(!(file = fopen(fpath, "r"))) {
        fprintf(stderr, "Error opening signature file:%s\n", fpath);
        exit(1);
    }
    
    if(stat(fpath, &fs)!=0) {
        fprintf(stderr, "Error stat() file:%s\n", fpath);
        exit(1);
    }
    
    fp->h_statuscode_res = (char *) malloc(fs.st_size + 10);
    fp->h_statuscode_res[0] = '\0';
    
    // search result
    while(fgets(buffer, MIN_ARG-1, file)) {
        c = buffer;
        d = strstr(buffer, ";");
        if(d==NULL) {
            continue;
        }
        
        *d = '\0';
        d++;
        
        if(strcmp(fp->h_statuscode, d)==0) {
            myncat(fp->h_statuscode_res, c, fs.st_size -1);
            myncat(fp->h_statuscode_res, ",", fs.st_size -1);
        }
        
        c = NULL;
        d = NULL;
    }
    
    fclose(file);
}

/*
 * Analize HTTP status text
 */
void finger_status_text(struct fingerp *fp, const char *path)
{
    FILE *file;
    char fpath[MAX_DATA];
    char buffer[MIN_ARG];
    char *c=NULL, *d=NULL;
    struct stat fs;
    
    snprintf(fpath, MAX_DATA-1, "%s%s", path, FDB_STATUSTEXT);
    
    if(!(file = fopen(fpath, "r"))) {
        fprintf(stderr, "Error opening signature file:%s\n", fpath);
        exit(1);
    }
    
    if(stat(fpath, &fs)!=0) {
        fprintf(stderr, "Error stat() file:%s\n", fpath);
        exit(1);
    }
    
    fp->h_statustext_res = (char *) malloc(fs.st_size + 10);
    fp->h_statustext_res[0] = '\0';
    
    // search result
    while(fgets(buffer, MIN_ARG-1, file)) {
        c = buffer;
        d = strstr(buffer, ";");
        if(d==NULL) {
            continue;
        }
        
        *d = '\0';
        d++;
        
        if(strcmp(fp->h_statustext, d)==0) {
            myncat(fp->h_statustext_res, c, fs.st_size -1);
            myncat(fp->h_statustext_res, ",", fs.st_size -1);
        }
        
        c = NULL;
        d = NULL;
    }
    
    fclose(file);
}

/*
 * Analize HTTP server banner
 */
void finger_banner(struct fingerp *fp, const char *path)
{
    FILE *file;
    char fpath[MAX_DATA];
    char buffer[MIN_ARG];
    char *c=NULL, *d=NULL;
    struct stat fs;
    
    snprintf(fpath, MAX_DATA-1, "%s%s", path, FDB_BANNER);
    
    if(!(file = fopen(fpath, "r"))) {
        fprintf(stderr, "Error opening signature file:%s\n", fpath);
        exit(1);
    }
    
    if(stat(fpath, &fs)!=0) {
        fprintf(stderr, "Error stat() file:%s\n", fpath);
        exit(1);
    }
    
    fp->h_banner_res = (char *) malloc(fs.st_size + 10);
    fp->h_banner_res[0] = '\0';
    
    // search result
    while(fgets(buffer, MIN_ARG-1, file)) {
        c = buffer;
        d = strstr(buffer, ";");
        if(d==NULL) {
            continue;
        }
        
        *d = '\0';
        d++;
        
        if(strcmp(fp->h_banner, d)==0) {
            myncat(fp->h_banner_res, c, fs.st_size -1);
            myncat(fp->h_banner_res, ",", fs.st_size -1);
        }
        
        c = NULL;
        d = NULL;
    }
    
    fclose(file);
}

/*
 * Analize HTTP powered-by
 */
void finger_powered(struct fingerp *fp, const char *path)
{
    FILE *file;
    char fpath[MAX_DATA];
    char buffer[MIN_ARG];
    char *c=NULL, *d=NULL;
    struct stat fs;
    
    snprintf(fpath, MAX_DATA-1, "%s%s", path, FDB_POWERED);
    
    if(!(file = fopen(fpath, "r"))) {
        fprintf(stderr, "Error opening signature file:%s\n", fpath);
        exit(1);
    }
    
    if(stat(fpath, &fs)!=0) {
        fprintf(stderr, "Error stat() file:%s\n", fpath);
        exit(1);
    }
    
    fp->h_powered_res = (char *) malloc(fs.st_size + 10);
    fp->h_powered_res[0] = '\0';
    
    // search result
    while(fgets(buffer, MIN_ARG-1, file)) {
        c = buffer;
        d = strstr(buffer, ";");
        if(d==NULL) {
            continue;
        }
        
        *d = '\0';
        d++;
        
        if(strcmp(fp->h_powered, d)==0) {
            myncat(fp->h_powered_res, c, fs.st_size -1);
            myncat(fp->h_powered_res, ",", fs.st_size -1);
        }
        
        c = NULL;
        d = NULL;
    }
    
    fclose(file);
}

/*
 * Wrapper for "Analize*" functions
 */
void finger_wrapper(struct fingerp *fp, char *path)
{
    char *c, *e, *f;
    int i;
    
    /* HTTP status */
    if((c = strstr(fp->headers, " "))==NULL) {
        return;
    }
    c++;
    
    if((e = strstr(c, " "))==NULL) {
        return;
    }
    
    if((f = strstr(e, "\r\n"))==NULL) {
        return;
    }
    
    // status code
    *e = '\0';
 
    strncpy(fp->h_statuscode, c, MIN_ARG-1);
    myncat(fp->h_statuscode, "\r\n", MIN_ARG-1); 
    
    *e = ' ';
    
    finger_status_code(fp, path);
    
    // status text
    e++;
    *f = '\0';
    
    strncpy(fp->h_statustext, e, MAX_ARG-1); 
    myncat(fp->h_statustext, "\r\n", MAX_ARG-1);
    
    *f = '\r';
    
    finger_status_text(fp, path);
    
    
    /* HTTP headers */
    
    if((c = strstr(fp->headers, "\r\n"))==NULL) {
        return;
    }
       
    c += 2;

    while(1) {
        
        // e = c. Header first char
        e = c;
        
        // e = end of header name
        e = strstr(c, ":");
        if(e==NULL) break;
        
        // add header name to fp->order
        *e = '\0';
        myncat(fp->order, c, MAX_ARG-1);
        myncat(fp->order, ",", MAX_ARG-1);
        
        // fill fingerp struct
        if(strcmp(c, "Server")==0) { // server banner
            *e = ':';
            e++;
            while(*e == ' ') e++;
            
            if((f = strstr(e, "\r\n"))==NULL) {
                break;
            }
            
            *f = '\0';
            
            strncpy(fp->h_banner, e, MAX_ARG-1);
            myncat(fp->h_banner, "\r\n", MAX_ARG-1);
            
            *f = '\r';
            
            finger_banner(fp, path);
        }
        
        else if(strcmp(c, "X-Powered-By")==0) { // powered-by header
            *e = ':';
            e++;
            while(*e == ' ') e++;
            
            if((f = strstr(e, "\r\n"))==NULL) {
                break;
            }
            
            *f = '\0';
            
            strncpy(fp->h_powered, e, MAX_ARG-1);
            myncat(fp->h_powered, "\r\n", MAX_ARG-1);
            
            *f = '\r';
            
            finger_powered(fp, path);
        }
        
        else {
            *e = ':';
        }
        
        if((c = strstr(c, "\r\n"))==NULL) {
            break;
        }
       
        c += 2;
    }
    
    /* HTTP header order */
    i = 0;
    while(fp->order[i]!='\0') i++;
    if(fp->order[i-1]==',') fp->order[i-1] = '\0';
    
    myncat(fp->order, "\r\n", MAX_ARG-1);
    
    finger_order(fp, path);
    
    return;
}

/*
 * Main fingerprint function
 */
void fingerprint(struct iprange *ipr, char *headers, char *buff, size_t buff_size)
{
    struct fingerp fp;
    struct stat buf;
    char dbpath[MAX_DATA];
    char *results;
    int i, y, count, res_count, res_size;
    char **resv, *result;
    
    memset(&fp, 0, sizeof(struct fingerp));
    
    // find database directory
    if(stat(DATABASE_PATH, &buf)==0) {
        strncpy(dbpath, DATABASE_PATH, MAX_DATA-1);
    }
    else if(stat(DATABASE_LOCAL, &buf)==0) {
        strncpy(dbpath, DATABASE_LOCAL, MAX_DATA-1);
    }
    else {
        fprintf(stderr, "\nError: fingerprint database directory not found.\n");
        exit(1);
    }
    
    // add headers to struct fingerp
    strncpy(fp.headers, headers, MAX_DATA-1);
    
    // append to dbpath the necessary subdirectory
    if(ipr->reqtype == GET_REQ) {
        myncat(dbpath, DB_GET, MAX_DATA-1);
        finger_wrapper(&fp, dbpath);
    }
    
    else if(ipr->reqtype == HEAD_REQ) {
        myncat(dbpath, DB_HEAD, MAX_DATA-1);
        finger_wrapper(&fp, dbpath);
    }
    
    else if(ipr->reqtype == GET_NOT_REQ) {
        myncat(dbpath, DB_GET_NOT, MAX_DATA-1);
        finger_wrapper(&fp, dbpath);
    }
    
    else if(ipr->reqtype == GET_WRONG_REQ) {
        myncat(dbpath, DB_GET_WRONG, MAX_DATA-1);
        finger_wrapper(&fp, dbpath);
    }
    
    else if(ipr->reqtype == OPTIONS_REQ) {
        myncat(dbpath, DB_OPTIONS, MAX_DATA-1);
        finger_wrapper(&fp, dbpath);
    }
    
    else if(ipr->reqtype == DELETE_REQ) {
        myncat(dbpath, DB_DELETE, MAX_DATA-1);
        finger_wrapper(&fp, dbpath);
    }
    
    // prepare results
    i = y = count = 0;

    res_size  = fp.order_res!=NULL ? strlen(fp.order_res) : 0;
    res_size += fp.h_statuscode_res!=NULL ? strlen(fp.h_statuscode_res) : 0;
    res_size += fp.h_statustext_res!=NULL ? strlen(fp.h_statustext_res) : 0;
    res_size += fp.h_banner_res!=NULL ? strlen(fp.h_banner_res) : 0;
    res_size += fp.h_powered_res!=NULL ? strlen(fp.h_powered_res) : 0;
    res_size += 10;
    
    results = (char *) malloc(res_size);
    results[0] = '\0';

    if(fp.order_res!=NULL) myncat(results, fp.order_res, res_size -1);
    if(fp.h_statuscode_res!=NULL) myncat(results, fp.h_statuscode_res, res_size -1);
    if(fp.h_statustext_res!=NULL) myncat(results, fp.h_statustext_res, res_size -1);
    if(fp.h_banner_res!=NULL) myncat(results, fp.h_banner_res, res_size -1);  
    if(fp.h_powered_res!=NULL) myncat(results, fp.h_powered_res, res_size -1);
    
    // count results
    if(results[0]!='\0' && results[0]!=',') {
        count++;
    }
    
    while(results[i]!='\0') {
        
        if(results[i]==',' && results[i+1]!=',' && results[i+1]!='\0') {
            count++;
        }
    
        i++;
    }
    
    // fill resv
    resv = (char **) malloc(count*sizeof(char *) + 1);
    
    i = y = count = 0;
    
    if(results[0]!='\0' && results[0]!=',') {
        resv[y] = results;
        y++;
    }
    
    
    while(results[i]!='\0') {
        
        if(results[i]==',' && results[i+1]!=',' && results[i+1]!='\0') {
            resv[y] = &results[i+1];
            results[i] = '\0';
            y++;
        }
    
        i++;
    }
    
    resv[y] = NULL;
    
    // display result
    i = y = count = res_count = 0;
    
    if(resv[0]==NULL) {
        printf("not found\n");
        return;
    }
    
    while(resv[y]!=NULL) {
        
        while(resv[i]!=NULL) {
            if(!strcmp(resv[y], resv[i])) {
                count++;
            }
        
            i++;
        }
        
        if(count > res_count) {
            result = resv[y];
            res_count = count;
        }
    
        y++;
        i = 0;
        count = 0;
    }
    
    // write result
    strncpy(buff, result, buff_size-1);
    
    free(results);
    free(resv);
    free_fingerp(&fp);
    
    return;
}

