/*
 * FINGERPRINT.H
 * Fingerprinting engine.
 *
 * Copyright (C) 2008  Acri Emanuele <crossbower@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef FINGERPRINT_LIB
#define FINGERPRINT_LIB

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

#include "argparser.h"
#include "httsquash.h"

#define DATABASE_LOCAL "./database/"
#define DATABASE_PATH "/usr/share/complemento/httsquash/database/"

#define DB_GET        "get_existing/"
#define DB_HEAD       "head_existing/"
#define DB_GET_NOT    "get_nonexisting/"
#define DB_GET_WRONG  "wrong_version/"
#define DB_OPTIONS    "options/"
#define DB_DELETE     "delete_existing/"

#define FDB_ORDER      "header-order.fdb"
#define FDB_STATUSCODE "statuscode.fdb"
#define FDB_STATUSTEXT "statustext.fdb"
#define FDB_BANNER     "banner.fdb"
#define FDB_POWERED    "x-powered-by.fdb"

/*
 * Fingerprint structure
 */
struct fingerp {
    char headers[MAX_DATA]; // full headers
    
    char order[MAX_ARG];    // order
    char *order_res;        // order result
    
    char h_statuscode[MIN_ARG]; // status code header
    char *h_statuscode_res;     // status code result
    
    char h_statustext[MAX_ARG]; // status text header
    char *h_statustext_res;     // status text result    
    
    char h_banner[MAX_ARG];  // banner header
    char *h_banner_res;      // banner result
    
    char h_powered[MAX_ARG]; // powered-by header
    char *h_powered_res;      // powered-by result 
};

/*
 * Free fingerp struct
 */
void free_fingerp(struct fingerp *fp);

/*
 * Analize HTTP header order
 */
void finger_order(struct fingerp *fp, const char *path);

/*
 * Analize HTTP status code
 */
void finger_status_code(struct fingerp *fp, const char *path);

/*
 * Analize HTTP status text
 */
void finger_status_text(struct fingerp *fp, const char *path);

/*
 * Analize HTTP server banner
 */
void finger_banner(struct fingerp *fp, const char *path);

/*
 * Analize HTTP powered-by
 */
void finger_powered(struct fingerp *fp, const char *path);

/*
 * Wrapper for "Analize*" functions
 */
void finger_wrapper(struct fingerp *fp, char *path);

/*
 * Main fingerprint function
 */
void fingerprint(struct iprange *ipr, char *headers, char *buff, size_t buff_size);

#endif /* FINGERPRINT_LIB */

