/*
 * HTTSQUASH.H
 *
 * Copyright (C) 2009  Acri Emanuele <crossbower@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef HTTSQUASH_LIB
#define HTTSQUASH_LIB

#include <unistd.h>
#include <ctype.h>
#include <signal.h>
#include <pthread.h>
#include <curl/curl.h>

#include "argparser.h"
#include "fingerprint.h"

/*
 * Structure for passing data to threads
 */
struct thread_data {
    struct iprange *ipr; // iprange pointer
    int curr;            // current host (last part, used in sprintf()...)
};


/*
 * Buffer for libcurl.
 * Taken from:
 * http://curl.haxx.se/lxr/source/docs/examples/getinmemory.c
 */
struct MemoryStruct {
    char *memory;
    size_t size;
};

/*
 * Write in memory (instead of file).
 * Taken from:
 * http://curl.haxx.se/lxr/source/docs/examples/getinmemory.c
 */
size_t WriteMemoryCallback(void *ptr, size_t size, size_t nmemb, void *data);

/*
 * Function called by scanloop()
 * Executed as a thread
 * Gets the ip address and retrieve informations from it
 */
void *scanstep(void *arg);

/*
 * Scan loop that assembles current ip address
 * and exec a callback function
 */
void scanloop(struct iprange *ipr, int max_th, void *p);

#endif /* HTTSQUASH_LIB */

